package nl.kallestruik.riseofblocks.blockentities;

import com.mojang.blaze3d.vertex.PoseStack;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.ChatFormatting;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.MultiBufferSource;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.client.renderer.block.BlockRenderDispatcher;
import net.minecraft.client.renderer.blockentity.BlockEntityRenderer;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.world.scores.Team;

import javax.annotation.ParametersAreNonnullByDefault;

public class FlagBlockRenderer implements BlockEntityRenderer<FlagBlockEntity> {
    @Override
    @ParametersAreNonnullByDefault
    public void render(FlagBlockEntity be, float v, PoseStack poseStack, MultiBufferSource multiBufferSource, int packedLight, int packedOverlay) {
        if (be.getLevel() == null) return;

        Team team = be.getLevel().getScoreboard().getPlayersTeam(be.getTeamName());
        int color = 0xFFFFFF; // default white

        if (team != null) {
            ChatFormatting textColor = team.getColor();
            color = textColor.getColor();
        }

        float r = ((color >> 16) & 0xFF) / 255f;
        float g = ((color >> 8) & 0xFF) / 255f;
        float b = (color & 0xFF) / 255f;

        poseStack.pushPose();

        // Translate to the block position center (optional)
        poseStack.translate(0.5, 0.5, 0.5);

        // Get model and vertex consumer
        BlockRenderDispatcher dispatcher = Minecraft.getInstance().getBlockRenderer();
        BakedModel model = dispatcher.getBlockModel(be.getBlockState());

        VertexConsumer vertexConsumer = multiBufferSource.getBuffer(RenderType.cutout());

        // Render model with tint
        dispatcher.getModelRenderer().renderModel(
                poseStack.last(),
                vertexConsumer,
                be.getBlockState(),
                model,
                r, g, b,
                packedLight,
                packedOverlay
        );

        poseStack.popPose();
    }
}
