package nl.kallestruik.riseofblocks.handlers;

import net.minecraft.core.BlockPos;
import net.minecraft.core.registries.Registries;
import net.minecraft.resources.ResourceKey;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.TickTask;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.scores.Team;
import net.neoforged.bus.api.SubscribeEvent;
import net.neoforged.fml.common.EventBusSubscriber;
import net.neoforged.neoforge.event.entity.player.PlayerEvent;
import net.neoforged.neoforge.event.level.BlockEvent;
import nl.kallestruik.riseofblocks.RiseOfBlocks;
import nl.kallestruik.riseofblocks.blockentities.FlagBlockEntity;
import nl.kallestruik.riseofblocks.registry.FlagRegistry;

@EventBusSubscriber(modid = RiseOfBlocks.MODID)
public class FlagEventHandler {

    @SubscribeEvent
    public static void onPlayerRespawn(PlayerEvent.Clone event) {
        System.out.println("onPlayerRespawn: " + event.getEntity().getScoreboardName() + " is " + (event.isWasDeath() ? "dead" : "alive") + " and is respawning.");
        if (!event.isWasDeath()) return; // Only handle actual death-respawns

        ServerPlayer player = (ServerPlayer) event.getEntity();
        ServerLevel world = player.serverLevel();
        MinecraftServer server = world.getServer();

        // Get player's team
        Team team = player.getScoreboard().getPlayersTeam(player.getScoreboardName());
        System.out.println("Team: " + (team == null ? "null" : team.getName()));
        if (team == null) return;

        // Get position from registry
        FlagRegistry.FlagPosition flagPos = FlagRegistry.get(world).get(team.getName());
        System.out.println("FlagPos: " + flagPos);
        if (flagPos == null || flagPos.getPos() == null) return;
        BlockPos pos = flagPos.getPos();
        ResourceKey<Level> dimensionKey = ResourceKey.create(Registries.DIMENSION, flagPos.getDimension());
        ServerLevel dimension = server.getLevel(dimensionKey);
        if (dimension == null) return;

        // Teleport after short delay (avoid spawn conflict)
        server.execute(() -> {
            player.teleportTo(dimension, pos.getX() + 0.5, pos.getY(), pos.getZ() + 0.5, player.getYRot(), player.getXRot());
        });
    }

    @SubscribeEvent
    public static void onBlockPlace(BlockEvent.EntityPlaceEvent event) {
        if (!(event.getLevel() instanceof ServerLevel world)) return;
        if (event.getEntity() instanceof Player player) {
            BlockPos pos = event.getPos();
            world.getServer().tell(new TickTask(1, () -> {
                BlockEntity be = world.getBlockEntity(pos);
                if (be instanceof FlagBlockEntity flagBe) {
                    Team team = player.getScoreboard().getPlayersTeam(player.getScoreboardName());
                    if (team != null) {
                        FlagRegistry flagRegistry = FlagRegistry.get(world);
                        FlagRegistry.FlagPosition oldFlagPos = flagRegistry.get(team.getName());
                        if (oldFlagPos != null) {
                            // Break the old flag if it exists
                            world.destroyBlock(oldFlagPos.getPos(), false, player);
                        }
                        flagBe.setTeamName(team.getName());
                        FlagRegistry.FlagPosition flagPos = new FlagRegistry.FlagPosition(pos, world.dimension().location());
                        flagRegistry.put(team.getName(), flagPos);
                    }
                }
            }));
        }
    }
}
